# -*- coding: utf-8 -*-
"""
slmd
~~~~

Sort lists in Markdown

:copyright: (c) 2017 by Park Hyunwoo.
:license: MIT, see LICENSE for more details.

"""

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
from collections import defaultdict

TAB_SIZE = 4
LIST_MARKERS = ('* ', '- ', '+ ')


def sort_bucket(bucket, sort_by=None, depth=0):
    order = 1

    if sort_by:
        try:
            order = sort_by[depth]
        except IndexError:
            order = 0

    if order == 1:
        bucket.sort()
    elif order == -1:
        bucket.sort(reverse=True)

    for item, child in bucket:
        if child:
            sort_bucket(child, sort_by, depth + 1)


def bake_bucket(output, bucket, depth=0):
    for item, child in bucket:
        output.write(item)
        if child:
            bake_bucket(output, child, depth + 1)


def sort(stream, output, sort_by=None):
    buckets = defaultdict(list)
    tab_positions = []
    last_depth = depth = 0

    for line in stream:
        stripped = line.lstrip()

        if not stripped:
            depth = 0
        else:
            if stripped[:2] in LIST_MARKERS:
                leading = len(line) - len(stripped)
                if not tab_positions:
                    if leading < TAB_SIZE:
                        # new list found
                        tab_positions.append(leading)
                        depth = 1
                else:
                    if leading in tab_positions:
                        depth = tab_positions.index(leading) + 1
                    else:
                        # new indent position found
                        if leading - tab_positions[-1] <= TAB_SIZE:
                            tab_positions.append(leading)
                            depth = len(tab_positions)

                # record latest depth for non-list items
                last_depth = depth
            else:
                # in list, but non-list item
                if tab_positions:
                    depth = last_depth + 1

        if depth > 0:
            buckets[depth] = []
            buckets[depth - 1].append((line, buckets[depth]))
        else:
            if buckets:
                sort_bucket(buckets[0], sort_by)
                bake_bucket(output, buckets[0])
                buckets.clear()
                del tab_positions[:]
            output.write(line)

    # for remains
    if buckets:
        sort_bucket(buckets[0], sort_by)
        bake_bucket(output, buckets[0])


def sort_string(string, sort_by=None):
    stream = StringIO(string)
    output = StringIO()
    sort(stream, output, sort_by)
    return output.getvalue()



import unittest
from slmd import sort_string


class TestSort(unittest.TestCase):

    def test_mandatory(self):
        self.assertEqual(True, True)

    def test_basic(self):
        asis = """
normal paragraph

- 2
  - C
  - A
    - 6
      indented text
    - 4
  - B
- 1
  - F
  - A

- k
- j

normal text
"""
        tobe = """
normal paragraph

- 1
  - A
  - F
- 2
  - A
    - 4
    - 6
      indented text
  - B
  - C

- j
- k

normal text
"""
        self.assertEqual(sort_string(asis), tobe)

    def test_descending(self):
        asis = """
* 1
* 3
* 2
"""
        tobe = """
* 3
* 2
* 1
"""
        self.assertEqual(sort_string(asis, [-1]), tobe)

    def test_1st_only(self):
        asis = """
* 1
  * 3
  * 2
  * 1
* 3
* 2
"""
        tobe = """
* 1
  * 3
  * 2
  * 1
* 2
* 3
"""
        self.assertEqual(sort_string(asis, [1]), tobe)

    def test_2nd_only(self):
        asis = """
* 1
  * 3
  * 2
  * 1
* 3
* 2
"""
        tobe = """
* 1
  * 1
  * 2
  * 3
* 3
* 2
"""
        self.assertEqual(sort_string(asis, [0, 1]), tobe)

    def test_3rd_only(self):
        asis = """
* 2
  * 3
  * 2
  * 1
    * B
    * A
    * C
* 3
* 1
"""
        tobe = """
* 2
  * 3
  * 2
  * 1
    * A
    * B
    * C
* 3
* 1
"""
        self.assertEqual(sort_string(asis, [0, 0, 1]), tobe)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import sys
import argparse
from slmd import sort_string


def get_parser():
    parser = argparse.ArgumentParser(description='Sort lists in Markdown')

    parser.add_argument('infile', type=argparse.FileType('r'),
                        default=sys.stdin)
    parser.add_argument('outfile', type=argparse.FileType('w'), nargs='?',
                        default=sys.stdout)
    parser.add_argument('-s', type=int, dest='sort_by', nargs='*',
                        help='Set order by depth (1:ASC / -1:DESC / 0:Do not sort)')

    return parser


def main():
    args = get_parser().parse_args()

    source = args.infile.read()
    result = sort_string(source, args.sort_by)

    args.outfile.write(result)

    sys.exit(0 if (source == result) else 1)


if __name__ == '__main__':
    main()



# slmd - Sort lists in Markdown

version_info = (0, 1, 1)

__version__ = VERSION = '.'.join(map(str, version_info))
__project__ = PROJECT = 'mdls'
__author__ = AUTHOR = "Park Hyunwoo <ez.amiryo@gmail.com>"


from .slmd import sort, sort_string



#!/usr/bin/env python
from setuptools import setup
from setuptools import find_packages
import re

with open('slmd/__init__.py') as f:
    data = re.search(r'\(\s*(\d*).\s*(\d*).\s*(\d)*\)', f.read())
    version = ".".join([data.group(1), data.group(2), data.group(3)])
assert version

classifiers = [
    'Topic :: Terminals',
    'Topic :: Utilities',
    'Environment :: Console',
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: MIT License',
]

setup(
    name='slmd',
    version=version,
    packages=find_packages(),
    zip_safe=True,
    author='Park Hyunwoo',
    author_email='ez.amiryo' '@' 'gmail.com',
    maintainer='Park Hyunwoo',
    maintainer_email='ez.amiryo' '@' 'gmail.com',
    url='http://github.com/lqez/slmd',
    description='Markdown list sorter',
    classifiers=classifiers,
    test_suite='slmd.test',
    entry_points={
        'console_scripts': [
            'slmd = slmd.cmd:main',
        ],
    },
)
